@@ -6,6 +6,7 @@
 #include <cstring> // memcpy
 
 #include <R_ext/Rdynload.h>
+#include <R_ext/Visibility.h>
 
 #include <rc/util.h>
 
@@ -19,6 +20,7 @@
 
 #include "R_interface_common.hpp"
 #include "R_interface_crossvalidate.hpp"
+#include "R_interface_rbart.hpp"
 #include "R_interface_sampler.hpp"
 
 using std::size_t;
@@ -285,8 +287,58 @@ namespace {
     pthread_mutex_unlock(&fitMutex);
     pthread_mutex_destroy(&fitMutex);
   }*/
-  
-#define DEF_FUNC(_N_, _F_, _A_) { _N_, reinterpret_cast<DL_FUNC>(&_F_), _A_ }
+
+}
+
+#if __cplusplus >= 202002L
+#  include <bit>
+#else
+
+namespace std {
+
+#  if __cplusplus >= 201103L
+#    include <type_traits>
+
+// From https://en.cppreference.com/w/cpp/numeric/bit_cast
+template <class To, class From>
+typename std::enable_if<
+  sizeof(To) == sizeof(From) &&
+  std::is_trivially_copyable<From>::value &&
+  std::is_trivially_copyable<To>::value,
+  To>::type
+// constexpr support needs compiler magic
+bit_cast(const From& src) noexcept
+{
+  static_assert(std::is_trivially_constructible<To>::value,
+    "This implementation additionally requires destination type to be trivially constructible");
+
+  To dst;
+  std::memcpy(&dst, &src, sizeof(To));
+  return dst;
+}
+
+#  else
+
+// We are only using this to cast function pointers, which are trivially copiable.
+// is_trivially_copyable is compiler specific and isn't worth trying to reimplement
+// in c++98.
+template <class To, class From>
+To
+bit_cast(const From& src)
+{
+  To dst;
+  std::memcpy(&dst, &src, sizeof(To));
+  return dst;
+}
+
+#  endif
+
+}
+
+#endif
+
+extern "C" {
+#define DEF_FUNC(_N_, _F_, _A_) { _N_, std::bit_cast<DL_FUNC>(&_F_), _A_ }
 
   static R_CallMethodDef R_callMethods[] = {
     DEF_FUNC("dbarts_create", create, 3),
@@ -327,6 +379,8 @@ namespace {
     // below: testing
     DEF_FUNC("dbarts_setSIMDInstructionSet", setSIMDInstructionSet, 1),
     DEF_FUNC("dbarts_getMaxSIMDInstructionSet", getMaxSIMDInstructionSet, 0),
+
+    DEF_FUNC("rbart_fitted", rbart_getFitted, 4),
     { NULL, NULL, 0 }
   };
 
@@ -337,7 +391,7 @@ namespace {
     DL_FUNC function;
   } C_CallMethodDef;
   
-#define DEF_FUNC(_N_, _F_) { _N_, reinterpret_cast<DL_FUNC>(&_F_) }
+#define DEF_FUNC(_N_, _F_) { _N_, std::bit_cast<DL_FUNC>(&_F_) }
   
   static C_CallMethodDef C_callMethods[] = {
     DEF_FUNC("createCGMPrior", dbarts_createCGMPrior),
@@ -396,6 +450,7 @@ namespace {
     
     DEF_FUNC("printInitialSummary", dbarts_printInitialSummary),
     DEF_FUNC("printTrees", dbarts_printTrees),
+    DEF_FUNC("getTrees", dbarts_getTrees),
     DEF_FUNC("setRNGState", dbarts_setRNGState),
     
     DEF_FUNC("runSampler", dbarts_runSampler),
@@ -421,10 +476,7 @@ namespace {
   
 #undef DEF_FUNC
   
-}
-
-extern "C" {
-  void R_init_dbarts(DllInfo* info)
+  void attribute_visible R_init_dbarts(DllInfo* info)
   {
     R_registerRoutines(info, NULL, R_callMethods, NULL, NULL);
     R_useDynamicSymbols(info, static_cast<Rboolean>(FALSE));
