@@ -56,9 +56,8 @@
 
 #define STANDARD_NORMAL_DEFAULT EXT_RNG_STANDARD_NORMAL_INVERSION
 
-#if defined(__GNUC__) && (\
-  (!defined(__clang__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))) || \
-  ( defined(__clang__) && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 7))))
+#if (defined(__clang__) && (__clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 7))) || \
+    (defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)))
 #  define SUPPRESS_DIAGNOSTIC 1
 #endif
 
@@ -247,8 +246,13 @@ ext_rng* ext_rng_createDefault(bool useNative)
   UNPROTECT(1);
   
 #ifdef SUPPRESS_DIAGNOSTIC
-#  pragma GCC diagnostic push
-#  pragma GCC diagnostic ignored "-Wswitch-enum"
+#  ifdef __clang__
+#    pragma clang diagnostic push
+#    pragma clang diagnostic ignored "-Wswitch-enum"
+#  else
+#    pragma GCC diagnostic push
+#    pragma GCC diagnostic ignored "-Wswitch-enum"
+#  endif
 #endif
   switch (algorithmType) {
     case EXT_RNG_ALGORITHM_KNUTH_TAOCP:
@@ -301,7 +305,11 @@ ext_rng* ext_rng_createDefault(bool useNative)
   }
   
 #ifdef SUPPRESS_DIAGNOSTIC
-#  pragma GCC diagnostic pop
+#  ifdef __clang__
+#    pragma clang diagnostic pop
+#  else
+#    pragma GCC diagnostic pop
+#  endif
 #endif
   
   int errorCode = ext_rng_setStandardNormalAlgorithm(result, stdNormalType, normalState);
@@ -501,6 +509,7 @@ int ext_rng_setSeed(ext_rng* generator, uint_least32_t seed)
     }
     break;
     case EXT_RNG_ALGORITHM_INVALID:
+    default:
     return EINVAL;
   }
   
@@ -572,9 +581,12 @@ void ext_rng_writeSerializedState(const ext_rng* generator, void* state)
     
     state = (void*) ((char*) state + sizeof(double));
     
-#if defined(__clang__) && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 5))
+#if defined(__clang__) && (__clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 5))
 #  pragma clang diagnostic push
 #  pragma clang diagnostic ignored "-Wtautological-compare"
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wtype-limits"
 #endif
     for (size_t i = 0; i < sizeof(double) % sizeof(int); ++i) {
       *((char*) state) = 0;
@@ -589,8 +601,10 @@ void ext_rng_writeSerializedState(const ext_rng* generator, void* state)
      *((char*) state) = 0;
      state = (void*) ((char*) state + 1);
   }
-#if defined(__clang__) && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 5))
+#if defined(__clang__) && (__clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 5))
 #  pragma clang diagnostic pop
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+#  pragma GCC diagnostic pop
 #endif
 }
 
@@ -704,14 +718,19 @@ static void validateSeed(ext_rng* generator, bool isFirstRun)
 #define KNUTH_CONSTANT         9.31322574615479e-10
 
 #ifdef SUPPRESS_DIAGNOSTIC
-#  pragma GCC diagnostic push
-#  pragma GCC diagnostic ignored "-Wuninitialized"
-#  if defined(__GNUC__) && !defined(__clang__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7))
-#    pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
+#  ifdef __clang__
+#    pragma clang diagnostic push
+#    pragma clang diagnostic ignored "-Wuninitialized"
+#  else
+#    pragma GCC diagnostic push
+#    pragma GCC diagnostic ignored "-Wuninitialized"
+#    if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7)
+#      pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
+#    endif
 #  endif
 #endif
 // guarantees results in (0, 1)
-inline static double truncateToUnitInterval(double x) {
+static inline double truncateToUnitInterval(double x) {
   if (x <= 0.0) return 0.5 * THIRTY_TWO_BIT_INVERSE;
   if ((1.0 - x) <= 0.0) return 1.0 - 0.5 * THIRTY_TWO_BIT_INVERSE;
   return x;
@@ -816,7 +835,11 @@ double ext_rng_simulateContinuousUniform(ext_rng* generator)
 }
 
 #ifdef SUPPRESS_DIAGNOSTIC
-#  pragma GCC diagnostic pop
+#  ifdef __clang__
+#    pragma clang diagnostic pop
+#  else
+#    pragma GCC diagnostic pop
+#  endif
 #endif
 
 /* ===================  Mersenne Twister ========================== */
